reAct_prompt = """
You are an AI Agent who works in a loop for complex tasks.

You follow the cycle of **Thought → Action → PAUSE → Observation → Thought → Action → PAUSE → Observation → Thought → ... → Answer**. 
Each message in conversation should contain only one role at a time, followed by **PAUSE**.

### Rules:
1. **Thought**: Reflect on how to solve the problem. Describe the approach you will take without performing any actions. 
2. **Action**: Execute the solution (e.g., calculations or retrieving information) and immediately return . Wait for the result in the **Observation** step. 
3. **Observation**: After receiving the tool result, report whether the solution is correct or needs adjustments. This should not contain Answer 
4. **Answer**: Provide the final answer only when the task is fully complete. 

### Example Session:

## Example Actions (the following actions are available only for this example):
- **calculate**: e.g., `calculate: 4.5 * 3.2`. Runs a Python calculation. 

## Agent Flow:
**user**: What is (2 * 256) + 235?

**assistant**: Thought: To solve this, I need to first perform the multiplication (2 * 256) and then add 235. 

**assistant**: Action: calculate: 2 * 256, Output: 512 **PAUSE**

**assistant**: Observation: The result of 2 * 256 is 512. The result is correct. 

**assistant**: Thought: Now I will add 235 to 512. 

**tool**: Action2: calculate: 512 + 235, Output:  747 **PAUSE**

**assistant**: Observation: The result of 512 + 235 is 747. The result is correct. Now I can provide final answer. 

**assistant**: Answer: The final result is 747.

---

Now it’s your turn:

Keep in Mind:
- Execute one step at a time (Thought or Action or Observation or Answer).
- User can see only the Answer. So provide clear Answer at the end.
- If task is not provided, break the loop and give Answer directly.

"""


from typing import Any
from vyzeai.prompts.react_prompt import reAct_prompt

class Agent:
    def __init__(self, llm, tools=None, max_iterations=25, react_prompt=None ) -> None:
        self.llm = llm
        self.tools = tools
        self.max_iterations = max_iterations

        if react_prompt:
            self.react_prompt = react_prompt
        else:
            self.react_prompt = reAct_prompt

        if self.tools:
            self.llm.tools = self.tools
        
        self.llm.memory = True
        self.llm.update_system_message(self.react_prompt)

    def __call__(self, prompt) -> Any:
        response = self._execute(prompt)
        return response

    def _execute(self, prompt):
        i = 0
        next_prompt = prompt
        while i<self.max_iterations:
            i += 1
            response = self.llm.run(next_prompt)
            print(response)
            print("\n------------------------------------------------------------------------\n")
            next_prompt = None
            if 'Answer' in response:
                return response
                


from vyzeai.models.openai import ChatOpenAI
from vyzeai.tools.prebuilt_tools import extract_relevant_sections_from_website, generate_images_and_add_to_blog, generate_video, youtube_transcript_loader, extract_audio_from_video, transcribe_audio, post_on_linkedin, send_email, generate_image_openai, upload_to_drive
from vyzeai.tools.base_tool import Tool
import warnings
warnings.filterwarnings("ignore")

class ResearchAgent:
    def __init__(self, api_key=None) -> None:
        web_tool = extract_relevant_sections_from_website()
        self.tools = [web_tool]

        self.llm = ChatOpenAI(tools=self.tools, api_key=api_key)

    def research(self, topic, url): 
        prompt1 = (
            f"Gather relavent information about topic from the website. "
            f"\nTopic: {topic} "
            f"\nWebsite: {url} "
        )
        context = self.llm.run(prompt1, system_message="You are a senior research agent.", return_tool_output=True)
        return context
    
class LinkedInAgent:
    def __init__(self, api_key=None) -> None:
        linkedin_tool = post_on_linkedin()
        dalle_tool = generate_image_openai()
        self.tools = [linkedin_tool, dalle_tool]

        self.llm = ChatOpenAI(tools=self.tools, api_key=api_key)

    def generate_linkedin_post(self, content):
        prompt1 = (
            "Create a LinkedIn post based on the following topic and blog. The post should be professional, engaging, and suitable for a LinkedIn audience. "
            "It should introduce the topic, provide a brief summary, and include a call-to-action if relevant. The text should be concise yet informative."
            f"Blog content:\n{content}\n\n"
            
            "Expected Output: A well-structured LinkedIn post(around 250 words)."
            "Note: Do not post it on LinkedIn."
        )
        content = self.llm.run(prompt1)

        prompt2 = (
            f"Generate image for the follwing LinkedIn post."
            f"\nLinkedIn post: {content}"
            "\nNote: "
            "Think of the image prompt as 'what you want to see in the final image.' "
            "For instance: 'The sky was a crisp (blue:0.3) and (green:0.8)' indicates a sky that is predominantly green with a hint of blue. "
            "The weights (e.g., 0.3 and 0.8) apply to all words in the prompt, guiding the emphasis of the colors and elements. "
            "Ensure that the image prompts avoid including any text, names of individuals, company names, logos, or other identifiable information. "
        )

        image_path = self.llm.run(prompt2)[0]
        return content, image_path

    def post_content_on_linkedin(self, token, post_content, image_path=None):
        prompt5 = (
            "Post the following content as linkedin post. "
            f"Post content: {post_content}"
            f"image path: {image_path}"
            f"linkedin token: {token}"
        )
        ack = self.llm.run(prompt5)[0]
        return ack
    
class EmailAgent:
    def __init__(self, api_key) -> None:
        email_tool = send_email()
        self.tools = [email_tool]

        self.llm = ChatOpenAI(api_key=api_key, tools=self.tools)

    def generate_email(self, context):
        pass

    def send_email(self, to_mail, subject, body, attachments=None, credentials_json_file_path = 'credentials.json', token_json_file_path='token.json'):
        prompt = (
            f"Send an email to {to_mail} "
            f"subject: {subject} "
            f"body: {body} "
            f"attachments: {attachments}"
            f"\n sender details: and token_json_file_path = {token_json_file_path} "
            "use only token file for sender details"
        )
        ack = self.llm.run(prompt)
        return ack
    
class GoogleDriveAgent:
    def __init__(self, api_key) -> None:
        drive_tool = upload_to_drive()
        self.tools = [drive_tool]
        self.llm = ChatOpenAI(api_key=api_key, tools=self.tools)

    def upload(self, file_path, file_name, parent_folder_id, service_account='service_account.json'):
        prompt = (
            "Upload the following file to google drive"
            f"file path: {file_path}"
            f"file name: {file_name}"
            f"parent folder id: {parent_folder_id}"
            f"service account file path: {service_account}"
        )
        ack = self.llm.run(prompt)
        return ack

class BlogAgent:
    def __init__(self, api_key=None) -> None:
        self.llm1 = ChatOpenAI(api_key=api_key)

        blog_img_tool = generate_images_and_add_to_blog()
        self.tools = [blog_img_tool]

        self.llm = ChatOpenAI(tools=self.tools, api_key=api_key)

    def write_blog_text(self, topic, url, context):
        prompt2 = (
            f"Write a comprehensive blog post based on the following details:\n\n"
            f"Topic: {topic}\n"
            f"Company's website: {url}\n"
            f"Summarized content from company's website: {context}\n\n"
            f"The blog should include an engaging introduction to topic, then detailed stections about how the company addresses the topic, "
            f"and a conclusion summarizing the key points. Structure the blog with clear headings, and write it in a conversational style. "
            f"Use <-IMAGE-> placeholder where image image is required.(total 1 image only) "
            f"Output the blog in markdown format, including a title, introduction, body sections, and conclusion. Write in a conversational style to engage readers. "
        )
        blog_text = self.llm1.run(prompt2, system_message="You are an effective Blog writer.")
        return blog_text
    #f"Use '<-IMAGE->' placeholders: one after the introduction and another at a point where it would enhance the narrative.(total 2 images)"

    def add_image_prompts(self,blog_content):
        prompt3 = (
            "Please replace all instances of '<-IMAGE->' with specific image prompts. "
            "Each image prompt should be enclosed within XML tag ('<image>'). "
            "Ensure that the image prompts avoid including any text, names of individuals, company names, logos, or other identifiable information. "
            "Think of the image prompt as 'what you want to see in the final image.' "
            "Provide a descriptive prompt that clearly defines the elements, colors, and subjects. "
            "For instance: 'The sky was a crisp (blue:0.3) and (green:0.8)' indicates a sky that is predominantly green with a hint of blue. "
            "The weights (e.g., 0.3 and 0.8) apply to all words in the prompt, guiding the emphasis of the colors and elements. "
            f"Blog: \n{blog_content}\n\n"
            "Expected Output: A complete blog with image prompt(total 1 image only) enclosed in <image> tags. save temporarily."
        )
        blog = self.llm1.run(prompt3)
        return blog
    #"While you may reduce the number of images, ensure that no two image prompts are identical."
    
    def add_images(self, blog_content):
        prompt4 = f"Use tool to generate images and add them to below blog. \n\n Blog: {blog_content}"
        final_blog = self.llm.run(prompt4)
        return final_blog

    def generate_blog(self, topic, url, content):
        self.topic = topic
        self.url = url
        blog_text = self.write_blog_text(topic, url, content)
        blog_content = self.add_image_prompts(blog_text)
        self.blog_content = blog_content
        print(blog_content)
        output = self.add_images(blog_content)
        self.image_path = output[-1][-1][0]

        return output
    
class VideoAgent:
    def __init__(self, api_key) -> None:
        self.llm1 = ChatOpenAI(api_key=api_key)
        web_tool = extract_relevant_sections_from_website()
        video_tool = generate_video()
        self.tools = [web_tool, video_tool]
        self.llm = ChatOpenAI(tools=self.tools)
    
    def generate_script(self, topic, context):
        prompt2 = (
            "Generate a video script with exactly two narration and image prompt pairs for the following topic, focusing on the company's expertise related to the topic. "
            "The script should contain around 200 words total. Start by explaining the topic and then highlight the company's role or expertise in relation to it. "
            "The Narration must start with topic name. "
            "Ensure that the image prompts do not include any text, names, logos, or other identifying features. "
            "Provide a descriptive image prompt that clearly defines elements, colors, and subjects. For instance, 'The sky was a crisp blue with green hues' is more descriptive than just 'blue sky'."
            f"\n\n**Topic:** \n{topic}\n\n"
            f"**Company Website content:** \n{context}\n\n"
            "Expected Output: 2 pairs of sentences. Enclose the narration in <narration> narration here </narration> tags and image prompts in <image> image prompt here </image> tags."
        )
        script = self.llm1.run(prompt2)
        return script

    def create_video(self, script):
        prompt3 = f"Create a video for the follwing script. \n\n video script: {script}"
        video = self.llm.run(prompt3)
        return video
    
    def generate_video(self, topic, content):
        script = self.generate_script(topic, content)
        # print(script)
        video = self.create_video(script)
        return video

class YTBlogAgent:
    def __init__(self, api_key) -> None:
        self.llm1 = ChatOpenAI(api_key=api_key)
        yt_tool = youtube_transcript_loader()
        blog_img_tool = generate_images_and_add_to_blog()
        self.tools = [yt_tool, blog_img_tool]
        self.llm = ChatOpenAI(tools=self.tools, api_key=api_key)

    def extract_transcript(self, url):
        prompt1 = f"Extract the text content from the youtube video. video url: {url}"
        context = self.llm.run(prompt1)
        return context

    def write_blog_text(self, url, context):
        prompt2 = (
            f"Given a youtube video content, write a comprehensive blog post. "
            f"Structure the blog with clear headings, and write it in a conversational style. "
            f"Use <-IMAGE-> placeholder where image image is required.(total 1 image only) "
            f"Output the blog in markdown format, including a title, introduction, body sections, and conclusion. Write in a conversational style to engage readers. "
            f"**\n\nYouTube video url: ** {url}"
            f"\n**YouTube video transcript: **{context}"
        )
        blog_text = self.llm1.run(prompt2, system_message="You are an effective Blog writer.")
        return blog_text

    def add_image_prompts(self,blog_content):
        prompt3 = (
            "Please replace all instances of '<-IMAGE->' with specific image prompts. "
            "Each image prompt should be enclosed within XML tag ('<image> promt here </image>'). "
            "Ensure that the image prompts avoid including any text, names of individuals, company names, logos, or other identifiable information. "
            "Think of the image prompt as 'what you want to see in the final image.' "
            "Provide a descriptive prompt that clearly defines the elements, colors, and subjects. "
            "For instance: 'The sky was a crisp (blue:0.3) and (green:0.8)' indicates a sky that is predominantly green with a hint of blue. "
            "The weights (e.g., 0.3 and 0.8) apply to all words in the prompt, guiding the emphasis of the colors and elements. "
            "While you may reduce the number of images, ensure that no two image prompts are identical."
            f"Blog: \n{blog_content}\n\n"
            "Expected Output: A complete blog with image prompt(total 1 image only) enclosed in <image> tags. save temporarily."
        )
        blog = self.llm1.run(prompt3)
        return blog
    
    def add_images(self, blog_content):
        prompt4 = f"Use tool to generate images and add them to below blog. \n\n Blog: {blog_content}"
        final_blog = self.llm.run(prompt4)
        return final_blog
    
    def generate_blog(self, url):
        self.url = url
        transcript = self.extract_transcript(url)
        blog_text = self.write_blog_text(url, transcript)
        blog_content = self.add_image_prompts(blog_text)
        self.blog_content = blog_content
        output = self.add_images(blog_content)
        self.image_path = output[-1][-1][0]
        return output
    
class VideoAudioBlogAgent:
    def __init__(self, api_key) -> None:
        import os
        os.environ['OPENAI_API_KEY'] = api_key
        self.llm1 = ChatOpenAI(api_key=api_key)
        video_to_audio_tool = extract_audio_from_video()
        audio_to_text_tool = transcribe_audio()
        blog_img_tool = generate_images_and_add_to_blog()
        self.tools = [video_to_audio_tool, audio_to_text_tool, blog_img_tool]
        self.llm = ChatOpenAI(tools=self.tools, api_key=api_key)
        self.video_formats = ["mp4", "avi", "mkv", "mov", "flv", "wmv", "webm", "mpeg", "mpg", "3gp", "m4v", "mxf", "vob", "ogv"]
        self.audio_formats = ["mp3", "wav", "aac", "flac", "ogg", "m4a"]

    def video_to_text(self, video_path):
        audio_path = self.llm.run(f"Extract audio fromt the video. video path: {video_path}", return_tool_output=True)[0]
        print(audio_path)
        text = self.llm.run(f"Transcribe the audio into text. audio_path: {audio_path}", return_tool_output=True)[0]
        return text

    def audio_to_text(self, audio_path):
        text = self.llm.run(f"Transcribe the audio into text. audio_path: {audio_path}", return_tool_output=True)[0]
        return text
    
    def extract_text(self, file_path):
        file_format = 'Audio file' if file_path.endswith('mp3') else 'Video file'
        if file_format == 'Video file':
            text = self.video_to_text(file_path)
        elif file_format == 'Audio file':
            text = self. audio_to_text(file_path)
        return text

    def write_blog_text(self, context):
        prompt2 = (
            f"Given transcription of a video, write a comprehensive blog post."
            f"The blog should include an engaging introduction to topic, then detailed stections about how the company addresses the topic, "
            f"and a conclusion summarizing the key points. Structure the blog with clear headings, and write it in a conversational style."
            f"Use <-IMAGE-> placeholder where image image is required.(total 1 image only) "
            f"Output the blog in markdown format, including a title, introduction, body sections, and conclusion. Write in a conversational style to engage readers. "
            f"\n\nVideo Transcript: {context}"
        )
        blog_text = self.llm1.run(prompt2, system_message="You are an effective Blog writer.")
        return blog_text

    def add_image_prompts(self,blog_content):
        prompt3 = (
            "Please replace all instances of '<-IMAGE->' with specific image prompts. "
            "Each image prompt should be enclosed within XML tag ('<image>'). "
            "Ensure that the image prompts avoid including any text, names of individuals, company names, logos, or other identifiable information. "
            "Think of the image prompt as 'what you want to see in the final image.' "
            "Provide a descriptive prompt that clearly defines the elements, colors, and subjects. "
            "For instance: 'The sky was a crisp (blue:0.3) and (green:0.8)' indicates a sky that is predominantly green with a hint of blue. "
            "The weights (e.g., 0.3 and 0.8) apply to all words in the prompt, guiding the emphasis of the colors and elements. "
            "While you may reduce the number of images, ensure that no two image prompts are identical."
            f"Blog: \n{blog_content}\n\n"
            "Expected Output: A complete blog with image prompt(total 1 image only) enclosed in <image> tags. save temporarily."
        )
        blog = self.llm1.run(prompt3)
        return blog
    
    def add_images(self, blog_content):
        prompt4 = f"Use tool to generate images and add them to below blog. \n\n Blog: {blog_content}"
        final_blog = self.llm.run(prompt4, return_tool_output=True)
        print(final_blog)
        return final_blog
    
    def generate_blog(self, file_path):
        self.file_path = file_path
        file_format = self._classify_file(file_path)
        if file_format == 'Video file':
            text = self.video_to_text(file_path)
        elif file_format == 'Audio file':
            text = self. audio_to_text(file_path)
        else:
            raise("Invalid file format.")
        print("1111111111111")
        blog_text = self.write_blog_text(text)
        print("1111111111111")
        blog_content = self.add_image_prompts(blog_text)
        print("1111111111111")
        self.blog_content = blog_content
        output = self.add_images(blog_content)
        print("Output: ", output)
        self.image_path = output[-1][-1][0]
        return output
    
    def _classify_file(self, file_path):
        video_extensions = ("mp4", "avi", "mkv", "mov", "flv", "wmv", "webm", "mpeg", "mpg", "3gp", "m4v", "mxf", "vob", "ogv")
        audio_extensions = ("mp3", "wav", "aac", "flac", "ogg", "m4a")
        
        if file_path.lower().endswith(video_extensions):
            return "Video file"
        elif file_path.lower().endswith(audio_extensions):
            return "Audio file"
        else:
            return "Unknown type"



import json

class ChatMemory:
    def __init__(self):
        self.memory = [{'role': 'system', 'content': '"You are a helpful assistant. If needed use appropriate tool wisely."'}]

    def add_message(self, role, content):
        """Adds a message with a specific role (user, assistant, system) to the memory."""
        self.memory.append({'role': role, 'content': content})

    def add_tool_message(self, role, content, tool_call_id):
        self.memory.append({'role': role, 'content': content, 'tool_call_id':tool_call_id})

    def update_system_message(self, content):
        """Updates the existing system message with new content."""
        if self.memory and self.memory[0]['role'] == 'system':
            self.memory[0]['content'] = content
        else:
            self.memory.insert(0, {'role': 'system', 'content': content})

    def get_memory(self):
        """Returns the entire chat memory."""
        return self.memory

    def clear_memory(self, keep_system_message=True):
        """Clears the memory. Keeps the system message by default unless keep_system_message is False."""
        if keep_system_message:
            self.memory = [{'role': 'system', 'content': 'You are a helpful assistant.'}]
        else:
            self.memory = []

    def last_message(self):
        """Returns the last message."""
        if self.memory:
            return self.memory[-1]
        return None

    def search_memory(self, keyword):
        """Returns all messages that contain the keyword."""
        return [msg for msg in self.memory if keyword.lower() in msg['content'].lower()]
    
    def add_response(self, response, tool_output=None):
        role, content, tool_call_ids = self.handle_response(response, tool_output)
        if tool_call_ids:
            self.memory.append(response.choices[0].message)
            for i, id in enumerate(tool_call_ids):
                self.add_tool_message(role, json.dumps(content[i]), id)
        else:
            self.add_message(role, content)
        return content
    
    def handle_response(self, response, tool_output=None):
        parsed_response = self.parse_response(response)[0]
        if parsed_response['finish_reason'] == 'stop':
            return parsed_response['role'], parsed_response['content'], None
        if parsed_response['finish_reason'] == 'function_call':
            role = 'tool'
            content = parsed_response['function_call'][0]
            return role, content
        if parsed_response['finish_reason'] == 'tool_calls':
            role = 'tool'
            # content = json.dumps({'function_name': parsed_response['tool_calls'][0].function.name, 'arguments': parsed_response['tool_calls'][0].function.arguments, 'output': tool_output})
            content = [{'function_name': tool_call.function.name, 'arguments':tool_call.function.arguments, 'output': tool_output[i]} for i, tool_call in enumerate(parsed_response['tool_calls'])]
            # print('ids', [tool_call.id for tool_call in parsed_response['tool_calls']])
            tool_call_ids = [tool_call.id for tool_call in parsed_response['tool_calls']]
            return role, content, tool_call_ids

    def parse_response(self, response):
        """Converts a ChatCompletion object to a dictionary with specific fields."""
        return [
            {
                'finish_reason': choice.finish_reason,
                'role': choice.message.role,
                'content': choice.message.content,
                'function_call': choice.message.function_call,
                'tool_calls': choice.message.tool_calls,
                'refusal': choice.message.refusal
            }
            for choice in response.choices
        ]




# from vyzeai.tools.raw_functions import *
from pydantic import BaseModel, Field, EmailStr, AnyUrl, FilePath, StringConstraints
from enum import Enum
from vyzeai.tools.base_tool import add_function, Tool
from typing import List, Optional, Annotated
import warnings
warnings.filterwarnings("ignore")

def extract_relevant_sections_from_website():
    try:
        from youtube_transcript_api import YouTubeTranscriptApi
        import re
        import requests
        from bs4 import BeautifulSoup
    except ImportError as e:
        raise ImportError(f"Required library for extract_relevant_sections_from_website is not installed: {e}. Please install it using `pip install youtube-transcript-api beautifulsoup4 requests`.")
    from vyzeai.tools.raw_functions import extract_relevant_sections_from_website as rf
    decorated_model = add_function(rf)(ExtractRelevantSectionsFromWebsite)
    tool = Tool(decorated_model)()
    return tool

#@add_function(extract_relevant_sections_from_website)
class ExtractRelevantSectionsFromWebsite(BaseModel):
    """This tool helps to extract specific sections from a website based on the given keywords."""
    url : AnyUrl = Field(description="URL of a website")
    keywords : List[str] = Field(description="A list of keywords(single word) used to find relevant content about a topic from a website. More keywords gives best result. ")
    class Config:
        protected_namespaces = ()

def post_on_twitter():
    try:
        import tweepy
    except ImportError as e:
        raise ImportError(f"Required library for post_on_twitter is not installed: {e}. Please install it using `pip install tweepy`.")
    from vyzeai.tools.raw_functions import post_on_twitter as rf
    decorated_model = add_function(rf)(PostOnTwitter)
    tool = Tool(decorated_model)()
    return tool

#@add_function(post_on_twitter)
class PostOnTwitter(BaseModel):
    """This tool helps to post a tweet on a specific Twitter account given their credentials."""
    tweet: str = Field(description="Twitter tweet content")
    consumer_key: str = Field(description="consumer_key - one of the four credentials")
    consumer_secret: str = Field(description="consumer_secret - one of the four credentials")
    access_token: str = Field(description="access_token - one of the four credentials")
    access_token_secret: str = Field(description="access_token_secret - one of the four credentials")

def post_on_linkedin():
    try:
        import requests
        import json
    except ImportError as e:
        raise ImportError(f"Required library for post_on_linkedin is not installed: {e}. Please install it using `pip install requests`.")
    from vyzeai.tools.raw_functions import post_on_linkedin as rf
    decorated_model = add_function(rf)(PostOnLinkedIn)
    tool = Tool(decorated_model)()
    return tool

#@add_function(post_on_linkedin)
class PostOnLinkedIn(BaseModel):
    """This tool helps to post on a specific LinkedIn account given his/her LinkedIn access token."""
    token: str = Field(description="LinkedIn access token")
    text_content: str = Field(description="LinkedIn post content")
    image_path: Optional[FilePath] = Field(default=None, description="Image path for the post")

def send_email():
    try:
        from email.mime.text import MIMEText
        import base64
        import os
        from google.auth.transport.requests import Request
        from google.oauth2.credentials import Credentials
        from google_auth_oauthlib.flow import InstalledAppFlow
        from email.mime.multipart import MIMEMultipart
    except ImportError as e:
        raise ImportError(f"Required library for send_email is not installed: {e}. Please install it using `pip install google-auth google-auth-oauthlib google-auth-httplib2`.")

    from vyzeai.tools.raw_functions import send_email as rf
    decorated_model = add_function(rf)(SendEmail)
    tool = Tool(decorated_model)()
    return tool

#@add_function(send_email)
class SendEmail(BaseModel):
    """This tool helps to send an email."""
    to_email: EmailStr = Field(description="Receiver email address")
    subject: str = Field(description="Email subject")
    body: str = Field(description="Email content")
    attachments: List = Field(default=None, description="A list of attachments(file paths) to send in mail, example: doc files, images, videos")
    credentials_json_file_path: Optional[FilePath] = Field(
        default='credentials.json', description="The file path to the JSON file containing the sender's email credentials."
    )
    token_json_file_path: Optional[FilePath] = Field(
        default='token.json', description="The file path to the JSON file containing the OAuth token for authentication."
    )

def upload_to_drive():
    try:
        from google.oauth2 import service_account
        from googleapiclient.discovery import build
        from googleapiclient.http import MediaFileUpload
    except ImportError as e:
        raise ImportError(f"Required library for upload_to_drive is not installed: {e}. Please install it using `pip install google-auth google-auth-oauthlib google-auth-httplib2 google-api-python-client`.")
    from vyzeai.tools.raw_functions import upload_to_drive as rf
    decorated_model = add_function(rf)(UploadToDrive)
    tool = Tool(decorated_model)()
    return tool

#@add_function(upload_to_drive)
class UploadToDrive(BaseModel):
    """This tool helps upload a file to Google Drive."""
    filepath: FilePath = Field(description="The path to the file to be uploaded.")
    filename: str = Field(description="The desired name for the file in Google Drive.")
    parent_folder_id: str = Field(description="The ID of the parent folder in Google Drive where the file will be uploaded.")
    service_account_file_path: Optional[FilePath] = Field(
        default='service_account.json', description="The path to the service account JSON file for authentication."
    )

def convert_md_to_docx():
    try:
        import pypandoc
    except ImportError as e:
        raise ImportError(f"Required library for convert_md_to_docx is not installed: {e}. Please install it using `pip install pypandoc`. Additionally, pypandoc requires Pandoc to be installed on your system")
    from vyzeai.tools.raw_functions import convert_md_to_docx as rf
    decorated_model = add_function(rf)(ConvertMDToDocx)
    tool = Tool(decorated_model)()
    return tool

#@add_function(convert_md_to_docx)
class ConvertMDToDocx(BaseModel):
    """Model for converting a Markdown file to a DOCX file."""
    md_file_path: FilePath = Field(description="The path to the Markdown file to be converted.")
    docx_file_path: FilePath = Field(description="The path where the converted DOCX file will be saved.")

class ModelName(Enum):
    """Enum for the available OpenAI models."""
    DALL_E_2 = "dall-e-2"
    DALL_E_3 = "dall-e-3"

class Quality(Enum):
    """Enum for the available image qualities."""
    STANDARD = "standard"
    HD = "hd"

def generate_image_openai():
    try:
        import tempfile
        from openai import OpenAI
        import requests
    except ImportError as e:
        raise ImportError(f"Required library for generate_image_openai is not installed: {e}. Please install it using `pip install requests openai`.")
    from vyzeai.tools.raw_functions import generate_image_openai as rf
    decorated_model = add_function(rf)(GenerateImageOpenAI)
    tool = Tool(decorated_model)()
    return tool

#@add_function(generate_image_openai)
class GenerateImageOpenAI(BaseModel):
    """Model for generating an image using OpenAI's image generation API."""
    text: str = Field(description="The text prompt for generating the image.")
    openai_api_key: Optional[str] = Field(description="The OpenAI API key for authentication.")
    model_name: Optional[ModelName] = Field(default=ModelName.DALL_E_2, description="The name of the OpenAI image generation model.")
    resolution: Optional[Annotated[str, StringConstraints(pattern=r'^\d+x\d+$')]] = Field(default="512x512", description="The resolution of the generated image, e.g., ['256x256', '512x512', '1024x1024', '1024x1792', '1792x1024'].")
    save_temp: Optional[bool] = Field(default=False, description="Whether to use a temporary file for the output image. Usually do not prefer saving temporary.")

# # class GenerateImageOpenAIModel(BaseModel):
# #     """Model for generating an image using OpenAI's image generation API."""
# #     text: str = Field(description="The text prompt for generating the image.")
# #     same_temp: Optional[bool] = Field(default=False, description="Whether to use a temporary file for the output image.")
# #     model_name: Optional[str] = Field(default="dall-e-2", description="The name of the OpenAI image generation model.")
# #     resolution: Optional[constr(regex=r'^\d+x\d+$')] = Field(default="512x512", description="The resolution of the generated image, e.g., '512x512'.")
# #     quality: Optional[str] = Field(default='standard', description="The quality of the generated image.")
# #     n: Optional[conint(ge=1)] = Field(default=1, description="The number of images to generate.")

def generate_images_and_add_to_blog():
    from vyzeai.tools.raw_functions import generate_images_and_add_to_blog as rf
    decorated_model = add_function(rf)(AddImagesToBlog)
    tool = Tool(decorated_model)()
    return tool

# @add_function(generate_images_and_add_to_blog)
class AddImagesToBlog(BaseModel):
    """This tool helps to generate images and add them to blog. Blog should contain image prompts in <image> prompt </image> tag."""
    blog_content : str = Field(description="content of a blog")
    save_temp : Optional[bool] = Field(False, description="if True, files and images are stored temporarily. ")

def generate_video():
    try:
        import tempfile
        from openai import OpenAI
        import requests
        import os
        import numpy as np
        import cv2
        from PIL import Image, ImageDraw, ImageFont
        from moviepy.editor import VideoFileClip, AudioFileClip, concatenate_videoclips, ImageClip
    except ImportError as e:
        raise ImportError(f"Required library for generate_video is not installed: {e}. Please install it using `pip install openai requests numpy opencv-python Pillow moviepy`.")
    from vyzeai.tools.raw_functions import generate_video as rf
    decorated_model = add_function(rf)(GenerateVideo)
    tool = Tool(decorated_model)()
    return tool

#@add_function(generate_video)
class GenerateVideo(BaseModel):
    """Given image prompt pairs and narration pairs, this tool helps to generate a video.
    Images are generated using image prompts and speech is generated using narration prompts."""
    pairs: str = Field(description="A string of narration and image prompt pairs enclosed in <narration> and <image> tags.")
    final_video_filename: Optional[str] = Field(default='video.mp4', description="Final video file name used to save the video.")

def extract_audio_from_video():
    try:
        from moviepy.editor import VideoFileClip
        import tempfile
    except ImportError as e:
        raise ImportError(f"Required library for extract_audio_from_video is not installed: {e}. Please install it using `pip install moviepy`.")
    from vyzeai.tools.raw_functions import extract_audio_from_video as rf
    decorated_model = add_function(rf)(ExtractAudioFromVideo)
    tool = Tool(decorated_model)()
    return tool

#@add_function(extract_audio_from_video)
class ExtractAudioFromVideo(BaseModel):
    """This tool is used for extracting audio from a video."""
    video_path: FilePath = Field(description="The path to the video file from which to extract audio.")

def transcribe_audio():
    try:
        from openai import OpenAI
    except ImportError as e:
        raise ImportError(f"Required library for transcribe_audio is not installed: {e}. Please install it using `pip install openai`.")
    from vyzeai.tools.raw_functions import transcribe_audio as rf
    decorated_model = add_function(rf)(TranscribeAudio)
    tool = Tool(decorated_model)()
    return tool

#@add_function(transcribe_audio)
class TranscribeAudio(BaseModel):
    """This tool is used for transcribing audio using OpenAI Whisper."""
    audio_file_path: FilePath = Field(description="The path to the audio file for transcription.")

def youtube_transcript_loader():
    try:
        from youtube_transcript_api import YouTubeTranscriptApi
    except ImportError as e:
        raise ImportError(f"Required library for youtube_transcript_loader is not installed: {e}. Please install it using `pip install youtube_transcript_api`.")
    from vyzeai.tools.raw_functions import youtube_transcript_loader as rf
    decorated_model = add_function(rf)(YouTubeTranscriptLoader)
    tool = Tool(decorated_model)()
    return tool

#@add_function(youtube_transcript_loader)
class YouTubeTranscriptLoader(BaseModel):
    """This tool helps to load transcript of a YouTube video."""
    url: AnyUrl = Field(description="YouTube video URL.")

def wikipedia_search():
    try:
        import wikipedia
        import wikipediaapi
    except ImportError as e:
        raise ImportError(f"Required library for Wikipedia search is not installed: {e}. Please install it using `pip install wikipedia wikipedia-api`.")
    from vyzeai.tools.raw_functions import wikipedia_search as rf
    decorated_model = add_function(rf)(WikipediaSearch)
    tool = Tool(decorated_model)()
    return tool

#@add_function(search_multiple_wikipedia_pages)
class WikipediaSearch(BaseModel):
    """Search multiple Wikipedia pages based on a query and return summaries or full content."""
    query: str = Field(..., description="Search term for querying Wikipedia.")
    lang: str = Field(default="en", description="Language of Wikipedia to query, default is 'en' for English.")
    result_count: int = Field(default=3, description="Number of search results to return, default is 3.")
    full_content: bool = Field(default=False, description="Whether to return full page content or just summary, default is summary.")

def calculate():
    from vyzeai.tools.raw_functions import calculate as rf
    decorated_model = add_function(rf)(Calculate)
    tool = Tool(decorated_model)()
    return tool

#@add_function(calculate)
class Calculate(BaseModel):
    """This tool is used to evaluate a mathematical expression."""
    operation: str = Field(..., description="Mathematical expression to evaluate (no symbols or text allowed).")

# @add_function(excel_to_sql)
class ExcelToSQL(BaseModel):
    """
    Model for storing Excel data into a SQL table.
    
    Attributes:
    - excel_file_path: Path to the Excel file.
    - table_name: Name of the SQL table where data will be stored.
    """
    excel_file_path: str = Field(..., description="Path to the Excel file")
    table_name: str = Field(..., description="Name of the table to store data")
    user: str = Field(..., description="Username for the MySQL connection")
    password: str = Field(..., description="Password for the MySQL connection")
    host: str = Field(..., description="Hostname or IP address of the MySQL server")
    db_name: str = Field(..., description="Database name")

def excel_to_sql():
    from vyzeai.tools.raw_functions import excel_to_sql as rf
    decorated_model = add_function(rf)(ExcelToSQL)
    tool = Tool(decorated_model)()
    return tool

# @add_function(execute_query)
class QueryExecution(BaseModel):
    """
    Model for executing a SQL query.
    
    Attributes:
    - query: The SQL query to execute.
    """
    query: str = Field(..., description="SQL query string to be executed")
    user: str = Field(..., description="Username for the MySQL connection")
    password: str = Field(..., description="Password for the MySQL connection")
    host: str = Field(..., description="Hostname or IP address of the MySQL server")
    db_name: str = Field(..., description="Database name")

    class Config:
        arbitrary_types_allowed = True

def execute_query():
    from vyzeai.tools.raw_functions import execute_query as rf
    decorated_model = add_function(rf)(QueryExecution)
    tool = Tool(decorated_model)()
    return tool


from setuptools import find_packages, setup

with open("src/vyzeai/README.md", "r") as f:
    long_description = f.read()

setup(
    name="vyzeai",
    version="0.1.04",
    description="A python package for generative ai",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/prudvireddyNS/vyzeai",
    author="Prudvi",
    author_email="prudhvisneha2003@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.10",
        "Operating System :: OS Independent",
    ],
    install_requires=['openai', 'requests', 'numpy', 'pydantic[email]'],
    extras_require={
        "dev": ["pytest>=7.0", "twine>=4.0.2"],
    },
    python_requires=">=3.10",
)